@@ -7,16 +7,16 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"reflect"
+	"slices"
 	"strings"
 	"time"
 )
 
 const (
 	// APIVersion is the default version of NGINX Plus API supported by the client.
-	APIVersion = 8
+	APIVersion = 9
 
 	pathNotFoundCode  = "PathNotFound"
 	streamContext     = true
@@ -25,7 +25,7 @@ const (
 )
 
 var (
-	supportedAPIVersions = versions{4, 5, 6, 7, 8}
+	supportedAPIVersions = versions{4, 5, 6, 7, 8, 9}
 
 	// Default values for servers in Upstreams.
 	defaultMaxConns    = 0
@@ -42,11 +42,14 @@ var ErrUnsupportedVer = errors.New("API version of the client is not supported b
 
 // NginxClient lets you access NGINX Plus API.
 type NginxClient struct {
-	version     int
+	apiVersion  int
 	apiEndpoint string
 	httpClient  *http.Client
+	checkAPI    bool
 }
 
+type Option func(*NginxClient)
+
 type versions []int
 
 // UpstreamServer lets you configure HTTP upstreams.
@@ -133,6 +136,7 @@ type Stats struct {
 	HTTPLimitRequests      HTTPLimitRequests
 	HTTPLimitConnections   HTTPLimitConnections
 	StreamLimitConnections StreamLimitConnections
+	Workers                []*Workers
 }
 
 // NginxInfo contains general information about NGINX Plus.
@@ -220,8 +224,21 @@ type HTTPRequests struct {
 // SSL represents SSL related stats.
 type SSL struct {
 	Handshakes       uint64
-	HandshakesFailed uint64 `json:"handshakes_failed"`
-	SessionReuses    uint64 `json:"session_reuses"`
+	HandshakesFailed uint64         `json:"handshakes_failed"`
+	SessionReuses    uint64         `json:"session_reuses"`
+	NoCommonProtocol uint64         `json:"no_common_protocol"`
+	NoCommonCipher   uint64         `json:"no_common_cipher"`
+	HandshakeTimeout uint64         `json:"handshake_timeout"`
+	PeerRejectedCert uint64         `json:"peer_rejected_cert"`
+	VerifyFailures   VerifyFailures `json:"verify_failures"`
+}
+
+type VerifyFailures struct {
+	NoCert           uint64 `json:"no_cert"`
+	ExpiredCert      uint64 `json:"expired_cert"`
+	RevokedCert      uint64 `json:"revoked_cert"`
+	HostnameMismatch uint64 `json:"hostname_mismatch"`
+	Other            uint64 `json:"other"`
 }
 
 // ServerZones is map of server zone stats by zone name
@@ -495,35 +512,79 @@ type HTTPLimitConnections map[string]LimitConnection
 // StreamLimitConnections represents limit connections related stats
 type StreamLimitConnections map[string]LimitConnection
 
-// NewNginxClient creates an NginxClient with the latest supported version.
-func NewNginxClient(httpClient *http.Client, apiEndpoint string) (*NginxClient, error) {
-	return NewNginxClientWithVersion(httpClient, apiEndpoint, APIVersion)
+// Workers represents worker connections related stats
+type Workers struct {
+	ID          int
+	ProcessID   uint64      `json:"pid"`
+	HTTP        WorkersHTTP `json:"http"`
+	Connections Connections
 }
 
-// NewNginxClientWithVersion creates an NginxClient with the given version of NGINX Plus API.
-func NewNginxClientWithVersion(httpClient *http.Client, apiEndpoint string, version int) (*NginxClient, error) {
-	if !versionSupported(version) {
-		return nil, fmt.Errorf("API version %v is not supported by the client", version)
+// WorkersHTTP represents HTTP worker connections
+type WorkersHTTP struct {
+	HTTPRequests HTTPRequests `json:"requests"`
+}
+
+// WithHTTPClient sets the HTTP client to use for accessing the API.
+func WithHTTPClient(httpClient *http.Client) Option {
+	return func(o *NginxClient) {
+		o.httpClient = httpClient
 	}
-	versions, err := getAPIVersions(httpClient, apiEndpoint)
-	if err != nil {
-		return nil, fmt.Errorf("error accessing the API: %w", err)
+}
+
+// WithAPIVersion sets the API version to use for accessing the API.
+func WithAPIVersion(apiVersion int) Option {
+	return func(o *NginxClient) {
+		o.apiVersion = apiVersion
 	}
-	found := false
-	for _, v := range *versions {
-		if v == version {
-			found = true
-			break
+}
+
+// WithCheckAPI sets the flag to check the API version of the server.
+func WithCheckAPI() Option {
+	return func(o *NginxClient) {
+		o.checkAPI = true
+	}
+}
+
+// NewNginxClient creates a new NginxClient.
+func NewNginxClient(apiEndpoint string, opts ...Option) (*NginxClient, error) {
+	c := &NginxClient{
+		httpClient:  http.DefaultClient,
+		apiEndpoint: apiEndpoint,
+		apiVersion:  APIVersion,
+		checkAPI:    false,
+	}
+
+	for _, opt := range opts {
+		opt(c)
+	}
+
+	if c.httpClient == nil {
+		return nil, fmt.Errorf("http client is not set")
+	}
+
+	if !versionSupported(c.apiVersion) {
+		return nil, fmt.Errorf("API version %v is not supported by the client", c.apiVersion)
+	}
+
+	if c.checkAPI {
+		versions, err := getAPIVersions(c.httpClient, apiEndpoint)
+		if err != nil {
+			return nil, fmt.Errorf("error accessing the API: %w", err)
+		}
+		found := false
+		for _, v := range *versions {
+			if v == c.apiVersion {
+				found = true
+				break
+			}
+		}
+		if !found {
+			return nil, fmt.Errorf("API version %v is not supported by the server", c.apiVersion)
 		}
 	}
-	if !found {
-		return nil, ErrUnsupportedVer
-	}
-	return &NginxClient{
-		apiEndpoint: apiEndpoint,
-		httpClient:  httpClient,
-		version:     version,
-	}, nil
+
+	return c, nil
 }
 
 func versionSupported(n int) bool {
@@ -553,7 +614,7 @@ func getAPIVersions(httpClient *http.Client, endpoint string) (*versions, error)
 		return nil, fmt.Errorf("%v is not accessible: expected %v response, got %v", endpoint, http.StatusOK, resp.StatusCode)
 	}
 
-	body, err := ioutil.ReadAll(resp.Body)
+	body, err := io.ReadAll(resp.Body)
 	if err != nil {
 		return nil, fmt.Errorf("error while reading body of the response: %w", err)
 	}
@@ -582,7 +643,7 @@ func createResponseMismatchError(respBody io.ReadCloser) *internalError {
 }
 
 func readAPIErrorResponse(respBody io.ReadCloser) (*apiErrorResponse, error) {
-	body, err := ioutil.ReadAll(respBody)
+	body, err := io.ReadAll(respBody)
 	if err != nil {
 		return nil, fmt.Errorf("failed to read the response body: %w", err)
 	}
@@ -794,7 +855,7 @@ func (client *NginxClient) get(path string, data interface{}) error {
 	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
 	defer cancel()
 
-	url := fmt.Sprintf("%v/%v/%v", client.apiEndpoint, client.version, path)
+	url := fmt.Sprintf("%v/%v/%v", client.apiEndpoint, client.apiVersion, path)
 
 	req, err := http.NewRequestWithContext(ctx, http.MethodGet, url, nil)
 	if err != nil {
@@ -812,7 +873,7 @@ func (client *NginxClient) get(path string, data interface{}) error {
 	}
 	defer resp.Body.Close()
 
-	body, err := ioutil.ReadAll(resp.Body)
+	body, err := io.ReadAll(resp.Body)
 	if err != nil {
 		return fmt.Errorf("failed to read the response body: %w", err)
 	}
@@ -828,7 +889,7 @@ func (client *NginxClient) post(path string, input interface{}) error {
 	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
 	defer cancel()
 
-	url := fmt.Sprintf("%v/%v/%v", client.apiEndpoint, client.version, path)
+	url := fmt.Sprintf("%v/%v/%v", client.apiEndpoint, client.apiVersion, path)
 
 	jsonInput, err := json.Marshal(input)
 	if err != nil {
@@ -860,7 +921,7 @@ func (client *NginxClient) delete(path string, expectedStatusCode int) error {
 	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
 	defer cancel()
 
-	path = fmt.Sprintf("%v/%v/%v/", client.apiEndpoint, client.version, path)
+	path = fmt.Sprintf("%v/%v/%v/", client.apiEndpoint, client.apiVersion, path)
 
 	req, err := http.NewRequestWithContext(ctx, http.MethodDelete, path, nil)
 	if err != nil {
@@ -885,7 +946,7 @@ func (client *NginxClient) patch(path string, input interface{}, expectedStatusC
 	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
 	defer cancel()
 
-	path = fmt.Sprintf("%v/%v/%v/", client.apiEndpoint, client.version, path)
+	path = fmt.Sprintf("%v/%v/%v/", client.apiEndpoint, client.apiVersion, path)
 
 	jsonInput, err := json.Marshal(input)
 	if err != nil {
@@ -1102,6 +1163,11 @@ func determineStreamUpdates(updatedServers []StreamUpstreamServer, nginxServers
 
 // GetStats gets process, slab, connection, request, ssl, zone, stream zone, upstream and stream upstream related stats from the NGINX Plus API.
 func (client *NginxClient) GetStats() (*Stats, error) {
+	endpoints, err := client.GetAvailableEndpoints()
+	if err != nil {
+		return nil, fmt.Errorf("failed to get stats: %w", err)
+	}
+
 	info, err := client.GetNginxInfo()
 	if err != nil {
 		return nil, fmt.Errorf("failed to get stats: %w", err)
@@ -1147,21 +1213,6 @@ func (client *NginxClient) GetStats() (*Stats, error) {
 		return nil, fmt.Errorf("failed to get stats: %w", err)
 	}
 
-	streamZones, err := client.GetStreamServerZones()
-	if err != nil {
-		return nil, fmt.Errorf("failed to get stats: %w", err)
-	}
-
-	streamUpstreams, err := client.GetStreamUpstreams()
-	if err != nil {
-		return nil, fmt.Errorf("failed to get stats: %w", err)
-	}
-
-	streamZoneSync, err := client.GetStreamZoneSync()
-	if err != nil {
-		return nil, fmt.Errorf("failed to get stats: %w", err)
-	}
-
 	locationZones, err := client.GetLocationZones()
 	if err != nil {
 		return nil, fmt.Errorf("failed to get stats: %w", err)
@@ -1182,11 +1233,51 @@ func (client *NginxClient) GetStats() (*Stats, error) {
 		return nil, fmt.Errorf("failed to get stats: %w", err)
 	}
 
-	limitConnsStream, err := client.GetStreamConnectionsLimit()
+	workers, err := client.GetWorkers()
 	if err != nil {
 		return nil, fmt.Errorf("failed to get stats: %w", err)
 	}
 
+	streamZones := &StreamServerZones{}
+	streamUpstreams := &StreamUpstreams{}
+	limitConnsStream := &StreamLimitConnections{}
+	streamZoneSync := &StreamZoneSync{}
+
+	if slices.Contains(endpoints, "stream") {
+		streamEndpoints, err := client.GetAvailableStreamEndpoints()
+		if err != nil {
+			return nil, fmt.Errorf("failed to get stats: %w", err)
+		}
+
+		if slices.Contains(streamEndpoints, "server_zones") {
+			streamZones, err = client.GetStreamServerZones()
+			if err != nil {
+				return nil, fmt.Errorf("failed to get stats: %w", err)
+			}
+		}
+
+		if slices.Contains(streamEndpoints, "upstreams") {
+			streamUpstreams, err = client.GetStreamUpstreams()
+			if err != nil {
+				return nil, fmt.Errorf("failed to get stats: %w", err)
+			}
+		}
+
+		if slices.Contains(streamEndpoints, "limit_conns") {
+			limitConnsStream, err = client.GetStreamConnectionsLimit()
+			if err != nil {
+				return nil, fmt.Errorf("failed to get stats: %w", err)
+			}
+		}
+
+		if slices.Contains(streamEndpoints, "zone_sync") {
+			streamZoneSync, err = client.GetStreamZoneSync()
+			if err != nil {
+				return nil, fmt.Errorf("failed to get stats: %w", err)
+			}
+		}
+	}
+
 	return &Stats{
 		NginxInfo:              *info,
 		Caches:                 *caches,
@@ -1205,9 +1296,30 @@ func (client *NginxClient) GetStats() (*Stats, error) {
 		HTTPLimitRequests:      *limitReqs,
 		HTTPLimitConnections:   *limitConnsHTTP,
 		StreamLimitConnections: *limitConnsStream,
+		Workers:                workers,
 	}, nil
 }
 
+// GetAvailableEndpoints returns available endpoints in the API.
+func (client *NginxClient) GetAvailableEndpoints() ([]string, error) {
+	var endpoints []string
+	err := client.get("", &endpoints)
+	if err != nil {
+		return nil, fmt.Errorf("failed to get endpoints: %w", err)
+	}
+	return endpoints, nil
+}
+
+// GetAvailableStreamEndpoints returns available stream endpoints in the API.
+func (client *NginxClient) GetAvailableStreamEndpoints() ([]string, error) {
+	var endpoints []string
+	err := client.get("stream", &endpoints)
+	if err != nil {
+		return nil, fmt.Errorf("failed to get endpoints: %w", err)
+	}
+	return endpoints, nil
+}
+
 // GetNginxInfo returns Nginx stats.
 func (client *NginxClient) GetNginxInfo() (*NginxInfo, error) {
 	var info NginxInfo
@@ -1340,7 +1452,7 @@ func (client *NginxClient) GetStreamZoneSync() (*StreamZoneSync, error) {
 // GetLocationZones returns http/location_zones stats.
 func (client *NginxClient) GetLocationZones() (*LocationZones, error) {
 	var locationZones LocationZones
-	if client.version < 5 {
+	if client.apiVersion < 5 {
 		return &locationZones, nil
 	}
 	err := client.get("http/location_zones", &locationZones)
@@ -1354,7 +1466,7 @@ func (client *NginxClient) GetLocationZones() (*LocationZones, error) {
 // GetResolvers returns Resolvers stats.
 func (client *NginxClient) GetResolvers() (*Resolvers, error) {
 	var resolvers Resolvers
-	if client.version < 5 {
+	if client.apiVersion < 5 {
 		return &resolvers, nil
 	}
 	err := client.get("resolvers", &resolvers)
@@ -1577,7 +1689,7 @@ func (client *NginxClient) UpdateStreamServer(upstream string, server StreamUpst
 
 // Version returns client's current N+ API version.
 func (client *NginxClient) Version() int {
-	return client.version
+	return client.apiVersion
 }
 
 func addPortToServer(server string) string {
@@ -1599,7 +1711,7 @@ func addPortToServer(server string) string {
 // GetHTTPLimitReqs returns http/limit_reqs stats.
 func (client *NginxClient) GetHTTPLimitReqs() (*HTTPLimitRequests, error) {
 	var limitReqs HTTPLimitRequests
-	if client.version < 6 {
+	if client.apiVersion < 6 {
 		return &limitReqs, nil
 	}
 	err := client.get("http/limit_reqs", &limitReqs)
@@ -1612,7 +1724,7 @@ func (client *NginxClient) GetHTTPLimitReqs() (*HTTPLimitRequests, error) {
 // GetHTTPConnectionsLimit returns http/limit_conns stats.
 func (client *NginxClient) GetHTTPConnectionsLimit() (*HTTPLimitConnections, error) {
 	var limitConns HTTPLimitConnections
-	if client.version < 6 {
+	if client.apiVersion < 6 {
 		return &limitConns, nil
 	}
 	err := client.get("http/limit_conns", &limitConns)
@@ -1625,7 +1737,7 @@ func (client *NginxClient) GetHTTPConnectionsLimit() (*HTTPLimitConnections, err
 // GetStreamConnectionsLimit returns stream/limit_conns stats.
 func (client *NginxClient) GetStreamConnectionsLimit() (*StreamLimitConnections, error) {
 	var limitConns StreamLimitConnections
-	if client.version < 6 {
+	if client.apiVersion < 6 {
 		return &limitConns, nil
 	}
 	err := client.get("stream/limit_conns", &limitConns)
@@ -1640,3 +1752,16 @@ func (client *NginxClient) GetStreamConnectionsLimit() (*StreamLimitConnections,
 	}
 	return &limitConns, nil
 }
+
+// GetWorkers returns workers stats.
+func (client *NginxClient) GetWorkers() ([]*Workers, error) {
+	var workers []*Workers
+	if client.apiVersion < 9 {
+		return workers, nil
+	}
+	err := client.get("workers", &workers)
+	if err != nil {
+		return nil, fmt.Errorf("failed to get workers: %w", err)
+	}
+	return workers, nil
+}
